// SelfTest.h : main header file for TSelfTest.
//

#if !defined(_SELF_TEST_INCLUDED_)
#define _SELF_TEST_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file for PCH
#endif

#include "AnalogIO.h"
#include "DigIO.h"
#include "XMLSetup.h"
#include "Utils/StorageCards.h"
#include "Utils/SSP.h"
#include "Utils/WatchdogTimer.h"
#include "Xpert/Win32Fix.h"

#undef DECL
#ifdef ENGINE_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif


///////////////////////////////////////////////////////////////////////////////
// TSelfTest
//

class DECL TSelfTest
{
   ////////////////////////////////
   // Interface
   ////////////////////////////////
   public:

   // Constructors and destructor.
   TSelfTest();
   virtual ~TSelfTest() {};

   // Values to control flow.
   enum PromptResults { eNo, eYes, eTimeout };
   
   // Run self-tests.
   void Run(TWatchdogTimer& wdt, bool bManual = false);

   // Read and write setup.
   bool ReadSetup(CXMLSetup& XMLSetup);
   bool ReadCommonSetup(CXMLSetup& XMLSetup);
   bool WriteSetup(CXMLSetup& XMLSetup);
   bool InitSetup();

   ////////////////////////////////
   // Implementation
   ////////////////////////////////
   protected:
   friend class CSelfTestDlg;
   friend class TSelfTestEntry;
   friend class CSelfTestEntryDlg;

   // Path of results file.
   const CString m_strFile;

   // Tags used in results file.
   const char* m_szStartTag;
   const char* m_szEndTag;
   const char* m_szResetTag;
   const char* m_szIterationTag;
   const char* m_szPassTag;
   const char* m_szFailureTag;

   // Setup parameters.
   bool m_bExternalLoopback;
   bool m_bCFCard;
   bool m_bSDCard;
   bool m_bUSBCard;
   bool m_bPowerConsumption;
   bool m_bXliteButton;
   bool m_bXliteIO;
   bool m_bEthernet;

   // Iteration counter tracks the number of test iterations since the most 
   // recent start or recovery.
   int m_iIteration;

   // Return true if results file exists.
   bool FileExists();

   // Return true if results file is terminated with an end entry, or is blank.
   bool FileTerminated();

   // Write start tag entry to results file.
   void WriteStartTag();

   // Execute next iteration of tests.
   void RunTests();

   // Display test results.
   void DisplayResults();

   // Prompt user whether to run another iteration.
   int GoAgain();

   // Write end tag to results file.
   void WriteEndTag();

   // Delete results file (upon confirmation from user).
   void DeleteResults();

   // Ask user whether to count interruption as reset.
   int AskCountReset();

   // Ask user whether to jump to display of results.
   int AskDisplayResults();
   
   // Write reset tag to results file.
   void WriteResetTag();
   
   // Write iteration counter tag to results file.
   void WriteIterationTag();

   // Parse results file to determine number of resets, failures, and iterations 
   // that have occurred since the last start entry.
   void ParseResults(int& iResets, int& iFailures, int& iIterations, TData& strResults);

   // When a failure occurs, the test routine calls this method to write details 
   // of the failure to the results file. 
   void WriteFailEntry(LPCTSTR tstr);

   // Verify communication with the RTC.
   void VerifyRTCCommunication();

   // Verify physical memory characteristics.
   void VerifyPhysicalMemory();

   // Verify size of flash disk.
   void VerifyFlashDisk();

   // Verify watchdog timer is incrementing properly.
   void VerifyWatchdog();

   // Verify com ports.
   void VerifyComPorts();
   void FindPorts(bool bUART[], bool bCom[], DWORD dwAddr[], CString& str);
   void VerifyDataLines(bool bCom[], DWORD dwAddr[], CString& str);
   void InternalLoopback(bool bCom[], DWORD dwAddr[], CString& str);
   void ExternalLoopback(bool bCom[], CString& str);

   // Verify I2C devices.
   void VerifyI2CDevices();

   // Verify storage card.
   void VerifyStorageCard(TStorageCards::ECardTypes ctType);

   // Enter low power mode for user to measure consumption.
   void PowerConsumptionTest(CWnd* pParent);

   // Test Xlite display-navigation buttons.
   void VerifyDisplayButtons();

   // Verify battery voltage.
   void VerifyBatteryVoltage();

   // Verify Xlite I/O modules.
   void VerifyXliteIO(HANDLE hStopEvent);
   AnalogIO* m_pAIO;
   DigitalIO* m_pDIO;
   void AnalogTest(int iTestNum, int iChannel, int iGain, bool bDiff, 
         double dTargetVoltage, double dMargin);
   void DigitalTest(int iTestNum, int iOutChan, int iInpChan, int iNumCounts, 
         int iLimit);

   // Verify ethernet. 
   void VerifyEthernet();
};


// Declaration for the one and only global self-test agent object.
extern "C" DECL TSelfTest SelfTest;


#endif // !defined(_SELF_TEST_INCLUDED_)
